﻿(function ($) {
    $.tranzify = {
        defaults: { //默认配置
            transitionWidth: 40,    //遮罩层每一小片段的宽度
            transitionHeight: '100%',   //遮罩层每一小片段高度
            containerID: 'overlay', //遮罩层id
            transitionType: 'venetian', //默认遮罩层动画效果
            prevID: 'prev', //上一条导航ID
            nextID: 'next', //下一条导航ID
            visibleClass: 'visible' //可见性class
        },
        //插件初始化操作
        createUI: function (config) {
            var imgLength = $(config.selector).find('img').length,
            prevA = $('<a></a>', {
                id: config.prevID,
                href: '#',
                html: '&laquo;',
                click: function (e) {
                    e.preventDefault();
                    showNext(true);
                }
            }).appendTo(config.selector),
            nextA = $('<a></a>', {
                id: config.nextID,
                href: '#',
                html: '&raquo;',
                click: function (e) {
                    e.preventDefault();
                    showNext();
                }
            }).appendTo(config.selector);

            $(config.selector).find('img').eq(0).addClass(config.visibleClass);

            //显示函数
            function showNext(flag) {
                //隐藏导航
                $(config.selector).find('a').css('display', 'none');
                //创建遮罩
                $.tranzify.createOverlay(config);

                //获取当前显示状态的图片
                var currImg = $('.' + config.visibleClass, $(config.selector));

                if (flag === true) {
                    //当前图片不是第一张图片
                    if (currImg.prev().filter('img').length > 0) {
                        //将上一张图片设置为可显示状态
                        currImg.removeClass(config.visibleClass).prev().addClass(config.visibleClass);
                    } else {
                        //设置最后一张图片为可显示状态
                        currImg.removeClass(config.visibleClass);
                        $(config.selector).find('img').eq(imgLength - 1).addClass(config.visibleClass);
                    }
                } else {
                    //当前图片不是最后一张图片
                    if (currImg.next().filter('img').length > 0) {
                        //将下一张图片设置为可显示状态
                        currImg.removeClass(config.visibleClass).next().addClass(config.visibleClass);
                    } else {
                        //设置第一张图片为可显示状态
                        currImg.removeClass(config.visibleClass);
                        $(config.selector).find('img').eq(0).addClass(config.visibleClass);
                    }
                }

                //运行遮罩效果
                $.tranzify.runTransition(config);
            }

            //设置循环函数
            config.interval = setInterval(showNext, (config.multi * 150) + 3000);

            //鼠标移动到对象上，移出循环函数；鼠标移出，添加循环函数
            $(config.selector).hover(function () {
                clearInterval(config.interval);
            }, function () {
                config.interval = setInterval(showNext, (config.multi * 150) + 3000);
            });
        },
        //创建遮罩层
        createOverlay: function (config) {
            //判断是否有没有运行完的补间动画，存在则立即结束动画，移出对象
            var transOverlay = $('#' + config.containerID);
            if (transOverlay) {
                transOverlay.stop(true, true);
                transOverlay.remove();
            }

            //div left的偏移量
            var posLeftMarker = 0,
            //div 背景position的偏移量
            bgHorizMarker = 0,
            //遮罩层总的包装对象
            overlay = $('<div></div>', {
                id: config.containerID
            });

            //循环，确定需要创造的片段数
            for (var i = 0; i < config.multi; i++) {
                //创造片段，每个片段只包含当前显示图片的一部分图片
                $('<div></div>', {
                    //设置宽度
                    width: config.transitionWidth,
                    //设置高度
                    height: config.transitionHeight,
                    css: {
                        //设置背景图片，来源就是当前处于显示状态的图片
                        backgroundImage: 'url(' + $('.' + config.visibleClass, $(config.selector)).attr('src') + ')',
                        //设置背景图片大小，让他们和外部容器高度、宽度一致。
                        //这样无论你图片的大小，都会和容易大小匹配
                        backgroundSize: $(config.selector).width() + 'px ' + $(config.selector).height() + 'px',
                        //设置背景偏移量
                        backgroundPosition: bgHorizMarker + 'px 0',
                        //设置left值
                        left: posLeftMarker,
                        top: 0
                    }
                }).appendTo(overlay); //添加到遮罩层容器

                //重新计算偏移量
                bgHorizMarker -= config.transitionWidth;
                posLeftMarker += config.transitionWidth;
            }
            //遮罩层容器添加到页面
            overlay.insertBefore('#' + config.prevID);
        },
        //运行动画效果
        runTransition: function (config) {
            //获取遮罩层容器
            var transOverlay = $('#' + config.containerID),
            //获取遮罩层容器下各div
            transEls = transOverlay.children(),
            len = transEls.length - 1;

            //根据配置运行不懂得动画效果
            switch (config.transitionType) {
                case 'venetian':
                    transEls.each(function (i) {
                        transEls.eq(i).animate({
                            width: 0
                        }, 'slow', function () {
                            if (i === len) {
                                transOverlay.remove();
                                $(config.selector).find('a').css('display', 'block');
                            }
                        });
                    });
                    break;
                case 'strip':
                    var counter = 0;
                    function strip() {
                        transEls.eq(counter).animate({
                            height: 0
                        }, 150, function () {
                            if (counter === len) {
                                transOverlay.remove();
                                $(config.selector).find("a").css("display", "block");
                            } else {
                                counter++;
                                strip();
                            }
                        });
                    }
                    strip();
                    break;
            }
        }
    };

    $.fn.extend({
        //创建插件函数
        tranzify: function (options) {
            //扩展配置
            var config = $.extend($.tranzify.defaults, options);
            //获取当前dom对象，传给config.selector
            config.selector = "#" + this.attr('id');
            //计算出我们需要创建的遮罩层片段数
            config.multi = parseInt(this.width()) / config.transitionWidth;
            //创建插件
            $.tranzify.createUI(config);
            //返回对象本身，实现链式效果
            return this;
        }
    });
})(jQuery);